///File : MainPanel.cpp
//------------------------------------------------------------------------------
/*
    This file is part of LCDnurse.

    LCDnurse is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    LCDnurse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with LCDnurse.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainPanel.hpp"
//------------------------------------------------------------------------------
MainPanel::MainPanel( wxWindow* parent, wxWindowID id, DataManager *Data, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style ), m_isRepairing(false), m_isBlack(false)
{
    // DataManager
    m_Data = Data;

    // Vars
    m_isDrawing = false;

    // Dialog
    m_ControlDialog = new ControlDialog(this, wxID_ANY, _T("LCD Nurse"), m_Data);
    m_ControlDialog->CenterOnScreen();
    m_ControlDialog->Show();

    // Pour capter les touches
	SetFocus();

	// Timer
	m_timer = new wxTimer(this);

    // Events
	Connect( wxID_ANY, wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::KeyDown));
	Connect( wxID_ANY, wxEVT_PAINT, wxPaintEventHandler(MainPanel::OnPaint));
	Connect( wxID_ANY, wxEVT_LEFT_DOWN, wxMouseEventHandler(MainPanel::OnClick));
	Connect( wxID_ANY, wxEVT_MOTION, wxMouseEventHandler(MainPanel::OnMouseMove));
	Connect( wxID_ANY, wxEVT_MOUSEWHEEL, wxMouseEventHandler(MainPanel::OnMouseWheel));

	Connect( m_timer->GetId(), wxEVT_TIMER, wxTimerEventHandler(MainPanel::OnTimer));
}
//------------------------------------------------------------------------------
void MainPanel::StartRepair()
{
    m_isRepairing = true;
    m_isDrawing = false;
    m_ControlDialog->Hide();
	m_timer->Start(m_Data->GetPeriod());
}
//------------------------------------------------------------------------------
void MainPanel::StopRepair()
{
    m_isRepairing = false;
	m_timer->Stop();
	Refresh();

    m_ControlDialog->MyRefresh();
    m_ControlDialog->Show(true);
}
//------------------------------------------------------------------------------
void MainPanel::OnTimer(wxTimerEvent &event)
{
    m_isBlack = ! m_isBlack;
    Refresh();
}
//------------------------------------------------------------------------------
void MainPanel::OnMouseWheel(wxMouseEvent &event)
{
    if(m_isRepairing)
    {
        return;
    }

    if(event.GetWheelRotation() > 0)
    {
        m_Data->NextBackgroundColor();
    }
    else if(event.GetWheelRotation() < 0)
    {
        m_Data->PreviousBackgroundColor();
    }

    Refresh();
    m_ControlDialog->MyRefresh();
}
//------------------------------------------------------------------------------
void MainPanel::OnMouseMove(wxMouseEvent &event)
{
    if(m_isDrawing && !m_isRepairing) // Si l'utilisateur déssine
    {
        RefreshRect(wxRect(m_beginPt, m_currentPt));
        m_currentPt = event.GetPosition();
        RefreshRect(wxRect(m_beginPt, m_currentPt));
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnClick(wxMouseEvent &event)
{
    if(m_isRepairing)
    {
        return;
    }

    if(!m_isDrawing)
    {
        wxString colorsTbl[4] = {COLORS_NAME_LIST_NO_BLACK};
        int color = wxGetSingleChoiceIndex(_("Please chose the pixel color..."), _("New dead pixel"), 4, colorsTbl);

        if(color != -1)
        {
            m_currentRectColor = color;
            m_beginPt = event.GetPosition();
            m_currentPt = event.GetPosition();
            m_isDrawing = true;
            RefreshRect(wxRect(m_beginPt, m_currentPt));
        }
    }
    else // Si l'utilisateur déssine
    {
        m_Data->AddRect(wxRect(m_beginPt, m_currentPt), m_currentRectColor);
        m_isDrawing = false;
        RefreshRect(wxRect(m_beginPt, m_currentPt));
        m_ControlDialog->MyRefresh();
    }

    event.Skip();
}
//------------------------------------------------------------------------------
void MainPanel::KeyDown(wxKeyEvent& event)
{
    if(event.GetKeyCode() == WXK_ESCAPE || event.GetKeyCode() == WXK_DELETE)
    {
        if(m_isDrawing) // Si l'utilisateur déssine : on annule
        {
            m_isDrawing = false;
            RefreshRect(wxRect(m_beginPt, m_currentPt));
        }
        else if(m_isRepairing)
        {
            StopRepair();
        }
        else if(event.GetKeyCode() == WXK_ESCAPE)
        {
            m_ControlDialog->MyRefresh();
            m_ControlDialog->Show(!m_ControlDialog->IsVisible());
        }
        else if(event.GetKeyCode() == WXK_DELETE)
        {
            wxRect refreshRect;
            if(m_Data->DeleteRect(event.GetPosition(), refreshRect))
            {
                RefreshRect(refreshRect);
                m_ControlDialog->MyRefresh();
            }
        }
    }
    else if(event.GetKeyCode() == WXK_SPACE)
    {
        m_Data->NextBackgroundColor();
        Refresh();
        m_ControlDialog->MyRefresh();
    }
}
//------------------------------------------------------------------------------------------------------------------------------
void MainPanel::SetBrush(const char &color, wxPaintDC &dc)
{
    switch(color)
    {
        case RED_CODE :
            dc.SetBrush(wxBrush(_T("RED"), wxSOLID ));
            break;
        case GREEN_CODE :
            dc.SetBrush(wxBrush(_T("GREEN"), wxSOLID ));
            break;
        case BLUE_CODE :
            dc.SetBrush(wxBrush(_T("BLUE"), wxSOLID ));
            break;
        case WHITE_CODE :
            dc.SetBrush(wxBrush(_T("WHITE"), wxSOLID ));
            break;
        case BLACK_CODE :
            dc.SetBrush(wxBrush(_T("BLACK"), wxSOLID ));
            break;
    }
}
//------------------------------------------------------------------------------------------------------------------------------
void MainPanel::OnPaint(wxPaintEvent& event)
{
    // Cfg
    wxPaintDC dc(this);
    dc.SetPen(*wxTRANSPARENT_PEN);

    // Background
    SetBrush(m_Data->GetBackgroundColor(), dc);
    dc.DrawRectangle(0, 0, dc.GetSize().GetWidth(), dc.GetSize().GetHeight());

    // Rect
    if(!m_isRepairing || (m_isRepairing && !m_isBlack))
    {
        std::vector<RectStr> rects = m_Data->GetRects();

        for(int i = 0; i < rects.size(); i++)
        {
            SetBrush(rects[i].color, dc);
            dc.DrawRectangle(rects[i].rect);
        }
    }

    // Current rect
    if(m_isDrawing)
    {
        SetBrush(m_currentRectColor, dc);
        dc.DrawRectangle(wxRect(m_beginPt, m_currentPt));
    }

    /*wxFont MaFont(20, wxSWISS , wxNORMAL, wxNORMAL, false, _T("Arial Black"));
    MonDc.SetFont(MaFont);
    MonDc.DrawRotatedText(_T("Bonjour"),0,100,45);

    wxBrush MaBrush(_T("RED"),wxSOLID );
    MonDc.SetBrush(MaBrush) ;
    MonDc.DrawCircle(300,70,50) ;

    wxColour MaCouleur(0,0,255);

    wxPen MonCrayon(MaCouleur,10,wxSOLID);
    MonDc.SetPen(MonCrayon);
    MonDc.DrawLine(120,20,220,120);
    MonDc.DrawLine(220,20,120,120);*/
}
//------------------------------------------------------------------------------
